@@ -9,21 +9,12 @@ from more_click import verbose_option
 from tabulate import tabulate
 
 from .api import cursor
+from .queries import ID_NAME_QUERY_EXAMPLE
 
 __all__ = [
     "main",
 ]
 
-QUERY = """
-SELECT
-    MOLECULE_DICTIONARY.chembl_id,
-    MOLECULE_DICTIONARY.pref_name
-FROM MOLECULE_DICTIONARY
-JOIN COMPOUND_STRUCTURES ON MOLECULE_DICTIONARY.molregno == COMPOUND_STRUCTURES.molregno
-WHERE molecule_dictionary.pref_name IS NOT NULL
-LIMIT 5
-"""
-
 
 @click.command()
 @verbose_option
@@ -31,8 +22,7 @@ LIMIT 5
 def main(version: Optional[str]):
     """Test the connection."""
     with cursor(version=version) as c:
-        click.echo(f"using cursor {c}")
-        c.execute(QUERY)
+        c.execute(ID_NAME_QUERY_EXAMPLE)
         click.echo(
             tabulate(
                 c.fetchall(),
